@@ -15,7 +15,7 @@ require 5.004;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.7.2';
+$VERSION = '0.7.3';
 
 ######################################################################
 
@@ -230,6 +230,8 @@ FormEngine also provides a set of functions for checking the form input, here to
 
 I<gettext> is used for international error message support. So use C<setlocale(LC_MESSAGES, 'german')> if you want to have german error messages (there isn't support for any other language yet, but it shouldn't be difficult to translate the .po file, don't hesitate!).
 
+Another usefull feature is the C<confirm> method, which forces the user to read through his input once again before submitting it.
+
 FormEngine is designed to make extension writing an easy task!
 
 =head1 OVERVIEW
@@ -301,9 +303,11 @@ Another possibility is to define a hash of hash references and pass a reference
 The meaning of the keys is explained below.
 You can call C<set_main_vars> for setting low level (main) variables as well, so the hash notation isn't necessary.
 
-=head2 The Default Skin
+=head2 The Default Skin (FormEngine)
 
-...knows the following field types:
+Normally it is no problem to call the same template with the same name several times in one form (except C<check_uniq>), but it might cause problems if you use the same name in diffrent templates.
+
+The following field types are known by the default skin:
 
 =over
 
@@ -327,12 +331,30 @@ B<select> - selection list in terms of a pull down menu (one can be selected)
 
 B<check> - selection list in terms of buttons (several can be selected)
 
+Using this template, you must give every option a diffrent name: option1, option2 ...
+
+The Advantage is, that it enables you to use the same option-list in one form several times. You might need that if you e.g. want to administrate several users at once.
+
+=item
+
+B<check_uniq> - selection list in terms of buttons (several can be selected)
+
+Using this template, you only have to give one name for all options, but you can't use this name again in the form. Normally, this doesn't matter and so you'll use this template instaed of C<check>.
+
 =item
 
 B<hidden> - invisible field(s), can be used for passing data
 
 =item
 
+B<print> - this template simply prints out the given value
+
+=item
+
+B<button> - displays a button
+
+=item
+
 B<emb_text> - text field, designed to be embedded (nested) in another template (see below)
 
 =back
@@ -382,7 +404,7 @@ B<TYPE> - if set to I<password> for each character a I<*> is printed (instead of
 
 =back
 
-These variables are available for all selection field types (C<radio>, C<select>, C<check>) only:
+These variables are available for selection field types (C<radio>, C<select>, C<check>) only:
 
 =over
 
@@ -410,6 +432,16 @@ B<ROWS> - the height of the text input area [default: 10]
 
 =back
 
+These variables are available for the C<button> field type only:
+
+=over
+
+=item
+
+B<TYPE> - can be 'button', 'submit' or 'reset' [default: 'button']
+
+=back
+
 These variables are so called I<main variables> they can be set by using the hash notation (see above) or by calling C<set_main_vars> (see below):
 
 =over
@@ -640,7 +672,7 @@ sub get {
 
 =head3 ok
 
-Returns true (1) when the form was submitted and no errors were found!
+Returns true (1) if the form was submitted and no errors were found!
 Else it returns false (0).
 This method simply calls C<is_submitted> and C<get_error_count>.
 
@@ -650,7 +682,7 @@ This method simply calls C<is_submitted> and C<get_error_count>.
 
 sub ok {
  my $self = shift;
- return $self->is_submitted && ! $self->get_error_count;
+ return $self->is_submitted && (! $self->get_error_count) && (! defined($self->{input}->{($self->{conf}->{CONFIRM_CANCEL} || $self->{default}->{main}->{CONFIRM_CANCEL})}));
 }
 
 ######################################################################
@@ -713,6 +745,50 @@ sub get_input_value {
 
 ######################################################################
 
+=head3 confirm ( [CONFIRMMSG] )
+
+Calling this method will print the users input data and ask him to klick 'Ok' or 'Cancel'. 'Ok' will submit the data once again and then C<is_confirmed> will return true (1). 'Cancel' will display the form, so that the user can change the data.
+
+By default the message defined in I<Config.pm> will be displayed, but you can also pass your own text.
+
+
+=cut
+
+######################################################################
+
+sub confirm {
+  my($self,$confirmsg) = @_;
+  my $tmp;
+
+  $self->{confirm} = 1;
+  $self->{conf}->{CONFIRMSG} = $confirmsg if($confirmsg);
+  $tmp = $self->{skin}->{main};
+  $self->{skin}->{main} = $self->{skin}->{$self->{confirm_skin}->{main}};
+  $self->make;
+  $self->{skin}->{main} = $tmp;
+}
+
+######################################################################
+
+=head3 is_confirmed
+
+This method returns true (1) when the form input was affirmed by the user
+(see C<confirm> method).
+
+=cut
+
+######################################################################
+
+sub is_confirmed {
+  my($self) = @_;
+  if(defined($self->{input}->{($self->{conf}->{CONFIRMED} || $self->{default}->{main}->{CONFIRMED})})) {
+    return 1;
+  }
+  return 0;
+}
+
+######################################################################
+
 =head2 Methods For Configuring FormEngine
 
 =head3 set_skin ( NAME )
@@ -844,6 +920,93 @@ sub add_checks {
 
 ######################################################################
 
+=head3 set_confirm_skin ( HASHREF )
+
+The confirm method replaces the form-templates with confirm-templates.
+The default replacements are defined in Config.pm. For Example the C<main> template is replaced by the C<confirm> template. The defaults are still taken from the original templates.
+
+The special key I<default> defines the template, which will be used when for a certain form-template no confirm-template was defined.
+
+So the referenced hash, which you must pass to this method, has to look like this:
+
+ FORM_TEMPLATE_NAME => CONFIRM_TEMPLATE_NAME,
+ FORM_TEMPLATE_NAME => CONFIRM_TEMPLATE_NAME,
+ .
+ .
+ .
+
+=cut
+
+######################################################################
+
+sub set_confirm_skin {
+  my ($self,$skin) = @_;
+  $self->{confirm_skin} = $skin and return 1 if(ref($skin) eq 'HASH');
+  return 0;
+}
+
+######################################################################
+
+=head3 add_confirm_skin ( HASHREF )
+
+With this method, you can update the current confirm skin settings.
+For more information see above.
+
+=cut
+
+######################################################################
+
+sub add_confirm_skin {
+  my ($self,$skin) = @_;
+  if(ref($skin) eq 'HASH') {
+    foreach $_ (keys(%$skin)) {
+      $self->{confirm_skin}->{$_} = $skin->{$_};
+    }
+    return 1;
+  }
+  return 0;
+}
+
+######################################################################
+
+=head3 set_confirm_handler ( HASHREF )
+
+Sometimes, it might make sense to call a special confirm handler instead of the really referenced one.
+So the referenced hash must assign a handler method to a handler- or template name.
+
+=cut
+
+######################################################################
+
+sub set_confirm_handler {
+  my ($self,$hashref) = @_;
+  $self->{confirm_handler} = $hashref and return 1 if(ref($hashref) eq 'HASH');
+  return 0;
+}
+
+######################################################################
+
+=head3 add_confirm_handler ( HASHREF )
+
+With this method, you can update the current confirm handler settings.
+For more information see above.
+
+=cut
+
+######################################################################
+
+sub add_confirm_handler {
+  my ($self,$hashref) = @_;
+  if(ref($hashref) eq 'HASH') {
+    foreach $_ (keys(%$hashref)) {
+      $self->{confirm_handler}->{$_} = $hashref->{$_};
+    }
+  }
+  return 0;
+}
+
+######################################################################
+
 =head2 Debug Methods
 
 =head3 set_debug ( DEBUGLEVEL )
@@ -976,7 +1139,7 @@ The form definition of hobbies.cgi:
     my @form = (
 	    {
 	      templ => 'check',
-	      NAME  => [['hobbies[1]','hobbies[2]'],['hobbies[3]','hobbies[4]'],['hobbies[5]','hobbies[6]'],'hobbies[7]','hobbies[8]'],
+	      NAME  => [['hobbies1','hobbies2'],['hobbies3','hobbies4'],['hobbies5','hobbies6'],'hobbies7','hobbies8'],
 	      TITLE => 'hobbies',
 	      OPTION => [['Parachute Jumping', 'Playing Video Games'], ['Doing Nothing', 'Soak'], ['Head Banging', 'Cat Hunting'], "Don't Know", '<&emb_text&>'],
 	      OPT_VAL => [[1,2], [3,4], [5,6], 7, 8],
@@ -1023,9 +1186,14 @@ sub _initialize {
   $self->{use_input} = 1;
   $self->{check_error} = 1;
   $self->{cont} = '';
+  $self->{conf} = {};
   $self->{call_make} = 0;
   $self->{call_before_make} = [];
 
+  $self->{confirm} = 0;
+  $self->{confirm_skin} = \%HTML::FormEngine::Config::confirm_skin;
+  $self->{confirm_handler} = \%HTML::FormEngine::Config::confirm_handler;
+
   # setting up the default skin
   # use:
   #  set_skin | add_skin | set_default | add_default | set_handler | add_handler
@@ -1079,9 +1247,14 @@ sub _check_conf {
   }
   elsif(ref($conf) eq 'HASH' && ref($conf->{sub}) eq 'ARRAY') {
     $tmp = $self->_check_conf($conf->{sub});
-    $conf->{sub} = $tmp->{sub};
-    $conf->{TEMPL} = $tmp->{TEMPL};
+    if(ref($tmp) eq 'HASH') {
+      $conf->{sub} = $tmp->{sub};
+      $conf->{TEMPL} = $tmp->{TEMPL};
+    }
   }
+  #else {
+  #  $conf = {};
+  #}
   return $conf;
 }
 
@@ -1130,14 +1303,19 @@ sub _parse {
   my $cont = shift;
   my $nconf_back = {};
   my (%itvars, $itval, $max, $i);
-  my ($handler, $templ, $args, $val, $tmp, $body, $pupo);
+  my ($handler, $templ, $args, $val, $tmp, @tmp, $body, $pupo,$res,@args);
   my ($cache,$default);
-
   # parsing handler calls, we mustn't touch <~ ~> blocks
   while($cont =~ m/^(?:(?:.*<~.*~>.*)*|(?:(?!.*<~.*).*))<&([a-z_]+)(?: (.*?))?&>/s) {
   ##while($cont =~ m/^(?=.*<~.*~>.*).*|(?!.*<~.*).*<&([a-z_]+)(?: ((?:(?!<&|&>)..)*.?))?&>/s) {
     $templ = $1;
     $args = $2;
+    @args = ();
+    if($args) {
+      push @args, '' if($args =~ m/^,$/);
+      push @args, split(',',$args);
+      push @args, '' if($args =~ m/,$/);
+    }
     $nconf_back = $self->{nconf};
     
     if(ref($self->{nconf}->{$templ}) eq 'ARRAY' && ref($self->{nconf}->{$templ}->[0]) eq 'HASH') {
@@ -1209,17 +1387,23 @@ sub _parse {
     if(! ($handler = $self->{handler}->{$templ})) {
       $handler = $self->{handler}->{default}
     }
-    # the following causes an segfault:
-    # $cont =~ s/<&$templ&>/&$handler($self,$templ)/e;
-    # this works:
-    $_ = &$handler($self,$args,$templ);
-    # replace subtemplate with result
-    
-    if($args) {
-      $cont =~ s/^((.*<~.*~>.*)*|((?!.*<~.*).*))<&$templ $args&>/$1.$_/esg;
+    if($self->{confirm}) {
+      $handler = $self->{confirm_handler}->{$templ} if($self->{confirm_handler}->{$templ});
+      $res = &$handler($self,($self->{confirm_skin}->{$templ} || $self->{confirm_skin}->{default}),@args);
     }
     else {
-      $cont =~ s/^((.*<~.*~>.*)*|((?!.*<~.*).*))<&$templ&>/$1$_/sg;
+      # the following causes an segfault:
+      # $cont =~ s/<&$templ&>/&$handler($self,$templ)/e;
+      # this works:
+      $res = &$handler($self,$templ,@args);
+      # replace subtemplate with result
+    }
+    $res = '' unless(defined($res));
+    if($args) {
+      while($cont =~ s/^((.*<~.*~>.*)*?|((?!.*<~.*).*?))<&$templ $args&>/$1$res/s){};
+    }
+    else {
+      while($cont =~ s/^((.*<~.*~>.*)*?|((?!.*<~.*).*?))<&$templ&>/$1$res/s){};
     }
     
     $self->{nconf} = $nconf_back;
@@ -1271,11 +1455,37 @@ sub _parse {
     $cont =~ s//$tmp/e;
   }
 
+  while($cont =~ m/<\!((?:(?:(?:(?!<\!|\!>)..)*.?<\!(?:(?!<\!|\!>)..)*.?\!>(?:(?!<\!|\!>)..)*.?)+)|(?:(?:(?!<\!)..)*.?))\!(?:([A-Z_ ]+)|([A-Z_\|]+))\!>/s) {
+    my $code = $1;
+    $tmp = 0;
+    if($2 or $2 eq '0') {
+      $_ = $2;
+      foreach $_ (split(' ',$_)) {
+	$_ = $self->_get_var($_,1);
+	++$tmp && $cont =~ s/// && last unless($_ or $_ eq '0');
+      }
+      $cont =~ s//$code/ if(!$tmp);
+    }
+    elsif($3 or $3 eq '0') {
+      $_ = $3;
+      foreach $_ (split('\|',$_)) {
+	$_ = $self->_get_var($_,1);
+	++$tmp && last if($_ or $_ eq '0');
+      }
+      $tmp ? $cont =~ s//$code/ : $cont =~ s///;
+    }
+  }
   # replace variables with theire values
   while($cont =~ m/<&(~)?([A-Z_]+)&>/) {
-    $cont =~ s//$self->_parse($self->_get_var($2,$1))/e;
+    $tmp = $&;
+    $res = $self->_parse($self->_get_var($2,$1));
+    $cont =~ s/$tmp/$res/gs;
   }
-  
+
+  #while($cont =~ s/<&(~)?([A-Z_]+)&>/$self->_parse($self->_get_var($2,$1))/ges) {
+  #  print STDERR "\nA", $cont, "B\n";
+  #}  
+
   return $cont;
 }
 
@@ -1432,6 +1642,51 @@ I am always interested in knowing how my work helps others, so if you put this m
 
 Address comments, suggestions, and bug reports to moritz@freesources.org. 
 
+=head1 CREDITS
+
+Special thanks go to Darren Duncan. His HTML::FormTemplate module gave me a good example how to write a documentation. There are several similarities between HTML::FormEngine and HTML::FormTemplate, we both came to an related API design, the internal processes are completly diffrent. It wasn't my purpose to have these api design decisions in common with HTML::FormTemplate. When i wrote the php version of HTML::FormEngine, i didn't know anything about HTML::FormTemplate. Later i just ported this php class to perl. I think we both came to an likewise API because its just the most obvious solution.
+
+Features which FormEngine has and FormTemplate hasn't:
+
+=over
+
+=item
+
+Skinsystem
+
+=item
+
+More flexible validation and error message report
+
+=item
+
+Common checking methods are predefined, others can be added on the fly
+
+=item
+
+Internationalization with help of gettext
+
+=item
+
+Adaptation through Config.pm
+
+=item
+
+Due to the handler system and the modular design FormEngine can easily
+be extended
+
+=item
+
+A flexible set of methods to let the user confirm his input
+
+=back
+
+Features which FormTemplate has and FormEngine hasn't:
+
+I<This list will be filled in later.>
+
+(I ask the author to send me some notes, he told me he'll do so at opportunity.)
+
 =head1 SEE ALSO
 
 HTML::FormTemplate by Darren Duncan
