@@ -55,6 +55,13 @@ You can nest loops. For example the I<text> template uses this feature: If you u
 the text fields are printed each on a single line, if you use two dimensional arrays, you can print several
 text fields on the same line.
 
+=head2 <! !> Blocks
+
+Code that is enclosed in '<! ... ! VARIABLENAMES !>' is only printed
+when all variables which are mentioned in VARIABLENAMES are defined
+(not empty). If you seperate the variable names by '|' instead of ' ',
+only one of these variables must be defined.
+
 =cut
 
 ######################################################################
@@ -73,6 +80,24 @@ $skin{main} = '
 </table>
 </form>
 ';
+
+$skin{confirm} = '
+<form action="<&ACTION&>" method="<&METHOD&>">
+<&CONFIRMSG&><br><br>
+<input type="hidden" name="<&FORMNAME&>" value="1" />
+<table border=0 align="center" summary=""><~
+<tr><&TEMPL&></tr>~TEMPL~>
+<tr>
+   <td align="left">
+     <input type="submit" name="<&CONFIRM_CANCEL&>" value="<&CANCEL&>">
+   </td>
+   <td align="right" colspan=2>
+      <input type="submit" value="<&SUBMIT&>" name="<&CONFIRMED&>" />
+   </td>
+</tr>
+</table>
+</form>
+';
   
 $skin{text} = '
    <td valign="top"><&TITLE&></td>
@@ -101,7 +126,7 @@ $skin{radio} = '
    <td>
       <table border=0 summary=""><~
         <tr><~
-          <td><input type="radio" value="<&OPT_VAL&>" name="<&NAME&>"<&checked&> /><&OPTION&></td>~OPTION OPT_VAL NAME~>
+          <td><input type="radio" value="<&OPT_VAL&>" name="<&NAME&>" <&checked&> /><&OPTION&></td>~OPTION OPT_VAL NAME~>
         </tr>~OPTION OPT_VAL NAME~>
       </table>
    </td>
@@ -112,7 +137,7 @@ $skin{select} = '
    <td valign="top"><&TITLE&></td>
    <td>
       <select size="<&SIZE&>" name="<&NAME&>"><~
-        <option value="<&OPT_VAL&>"<&checked selected&>><&OPTION&></option>~OPTION OPT_VAL~>
+        <option value="<&OPT_VAL&>" <&checked selected&>><&OPTION&></option>~OPTION OPT_VAL~>
       </select>
    </td>
    <td style="color:#FF0000" valign="bottom"><&error&></td>
@@ -124,7 +149,7 @@ $skin{check} = '
      <table summary=""><~
        <tr><~
          <td>
-           <input type="checkbox" value="<&OPT_VAL&>" name="<&NAME&>"<&checked&> /> <&OPTION&>
+           <!<input type="checkbox" value="<&OPT_VAL&>" name="<&NAME&>" <&checked&> />!OPT_VAL NAME!> <&OPTION&>
            <font style="color:#FF0000"><&error&></font>
          </td>~OPTION OPT_VAL NAME~>
        </tr>~OPTION OPT_VAL NAME~>
@@ -133,6 +158,45 @@ $skin{check} = '
    <td valign="bottom" style="color:#FF0000"></td>
 ';
 
+$skin{check_uniq} = '
+   <td valign="top"><&TITLE&></td>
+   <td>
+     <table summary=""><~
+       <tr><~
+         <td>
+           <!<input type="checkbox" value="<&OPT_VAL&>" name="<&NAME&>" <&checked_uniq&> />!OPT_VAL NAME!> <&OPTION&>
+           <font style="color:#FF0000"><&error&></font>
+         </td>~OPTION OPT_VAL NAME~>
+       </tr>~OPTION OPT_VAL NAME~>
+     </table>
+   </td>
+   <td valign="bottom" style="color:#FF0000"></td>
+';
+
+$skin{confirm_check} = '
+   <td valign="top"><&TITLE&></td>
+   <td>
+     <table summary="" border=0 cellspacing=0 cellpadding=0><~
+       <tr><~
+         <td><&confirm_checked&></td>~OPTION OPT_VAL NAME~>
+       </tr>~OPTION OPT_VAL NAME~>
+     </table>
+   </td>
+   <td valign="bottom" style="color:#FF0000"></td>
+';
+
+$skin{confirm_check_uniq} = '
+   <td valign="top"><&TITLE&></td>
+   <td>
+     <table summary="" border=0 cellspacing=0 cellpadding=0><~
+       <tr><~
+         <td><&confirm_checked_uniq&></td>~OPTION OPT_VAL NAME~>
+       </tr>~OPTION OPT_VAL NAME~>
+     </table>
+   </td>
+   <td valign="bottom" style="color:#FF0000"></td>
+';
+
 $skin{textarea} = '
    <td valign="top"><&TITLE&></td>
    <td><textarea name="<&NAME&>" cols="<&COLS&>" rows="<&ROWS&>"><&value&></textarea></td>
@@ -145,8 +209,41 @@ $skin{hidden} = '
    </td>
 ';
 
-$skin{emb_text} = '
-     <&TITLE&> <input type="<&TYPE&>" value="<&value NAME&>" name="<&NAME&>" maxlength="<&MAXLEN&>" size="<&SIZE&>" /> <font style="color:#FF0000" valign="bottom"><&error&></font>
+$skin{print} = '
+   <td valign="top"><&TITLE&></td>
+   <td>
+     <table border=0><~
+       <tr><~
+         <td><&value&><input type="hidden" name="<&NAME&>" value="<&value&>" /></td>~NAME VALUE~>
+       </tr>~NAME VALUE~>
+     </table>
+   </td>
+   <td style="color:#FF0000" valign="bottom"></td>
 ';
 
+$skin{default_confirm} = '
+   <td valign="top"><&TITLE&></td>
+   <td>
+     <table border=0><~
+       <tr><~
+         <td><&value -&><input type="hidden" name="<&NAME&>" value="<&value ,1&>" /></td>~NAME VALUE~>
+       </tr>~NAME VALUE~>
+     </table>
+   </td>
+   <td style="color:#FF0000" valign="bottom"></td>
+';
+
+$skin{emb_text} = '
+     <&TITLE&> <input type="<&TYPE&>" value="<&value&>" name="<&NAME&>" maxlength="<&MAXLEN&>" size="<&SIZE&>" /> <font style="color:#FF0000" valign="bottom"><&error&></font>
+';
+
+$skin{button} = '
+   <td colspan=3>
+     <table border=0><~
+       <tr><~
+         <td><input type="<&TYPE&>" name="<&NAME&>" value="<&TITLE&>" /></td>~NAME TITLE~>
+       </tr>~NAME TITLE~>
+     </table>
+   </td>';
+
 1;
